'use strict';

module.exports = function createStore(reducer, initialState) {
  var state = initialState || {};
  var listeners = [];
  var store = {
    getState: function getState(action) {
      return Object.assign({}, state, reducer(state, action || {}));
    },
    dispatch: function dispatch(action) {
      // for thunks
      if (typeof action === 'function') return action(store.dispatch, store.getState
      // for promises
      );if (Promise.resolve(action) === action) return Promise.resolve(action).then(store.dispatch);
      state = store.getState(action
      // first parameter is how to remove listener
      );listeners.forEach(function (listener, i) {
        return listener(function () {
          delete listeners[i];
        });
      });
    },
    subscribe: function subscribe(callback) {
      return listeners.push(callback);
    }
  };
  if (initialState) setImmediate(store.dispatch);
  return store;
};